# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/ecies_aead_hkdf.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/ecies_aead_hkdf.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tink.proto import common_pb2 as tink_dot_proto_dot_common__pb2
from tink.proto import tink_pb2 as tink_dot_proto_dot_tink__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n tink/proto/ecies_aead_hkdf.proto\x12\x12google.crypto.tink\x1a\x17tink/proto/common.proto\x1a\x15tink/proto/tink.proto\"\x98\x01\n\x12\x45\x63iesHkdfKemParams\x12\x39\n\ncurve_type\x18\x01 \x01(\x0e\x32%.google.crypto.tink.EllipticCurveType\x12\x34\n\x0ehkdf_hash_type\x18\x02 \x01(\x0e\x32\x1c.google.crypto.tink.HashType\x12\x11\n\thkdf_salt\x18\x0b \x01(\x0c\"G\n\x12\x45\x63iesAeadDemParams\x12\x31\n\x08\x61\x65\x61\x64_dem\x18\x02 \x01(\x0b\x32\x1f.google.crypto.tink.KeyTemplate\"\xc9\x01\n\x13\x45\x63iesAeadHkdfParams\x12:\n\nkem_params\x18\x01 \x01(\x0b\x32&.google.crypto.tink.EciesHkdfKemParams\x12:\n\ndem_params\x18\x02 \x01(\x0b\x32&.google.crypto.tink.EciesAeadDemParams\x12:\n\x0f\x65\x63_point_format\x18\x03 \x01(\x0e\x32!.google.crypto.tink.EcPointFormat\"x\n\x16\x45\x63iesAeadHkdfPublicKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x37\n\x06params\x18\x02 \x01(\x0b\x32\'.google.crypto.tink.EciesAeadHkdfParams\x12\t\n\x01x\x18\x03 \x01(\x0c\x12\t\n\x01y\x18\x04 \x01(\x0c\"}\n\x17\x45\x63iesAeadHkdfPrivateKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12>\n\npublic_key\x18\x02 \x01(\x0b\x32*.google.crypto.tink.EciesAeadHkdfPublicKey\x12\x11\n\tkey_value\x18\x03 \x01(\x0c\"Q\n\x16\x45\x63iesAeadHkdfKeyFormat\x12\x37\n\x06params\x18\x01 \x01(\x0b\x32\'.google.crypto.tink.EciesAeadHkdfParamsBb\n\x1c\x63om.google.crypto.tink.protoP\x01Z@github.com/tink-crypto/tink-go/v2/proto/ecies_aead_hkdf_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.ecies_aead_hkdf_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z@github.com/tink-crypto/tink-go/v2/proto/ecies_aead_hkdf_go_proto'
  _globals['_ECIESHKDFKEMPARAMS']._serialized_start=105
  _globals['_ECIESHKDFKEMPARAMS']._serialized_end=257
  _globals['_ECIESAEADDEMPARAMS']._serialized_start=259
  _globals['_ECIESAEADDEMPARAMS']._serialized_end=330
  _globals['_ECIESAEADHKDFPARAMS']._serialized_start=333
  _globals['_ECIESAEADHKDFPARAMS']._serialized_end=534
  _globals['_ECIESAEADHKDFPUBLICKEY']._serialized_start=536
  _globals['_ECIESAEADHKDFPUBLICKEY']._serialized_end=656
  _globals['_ECIESAEADHKDFPRIVATEKEY']._serialized_start=658
  _globals['_ECIESAEADHKDFPRIVATEKEY']._serialized_end=783
  _globals['_ECIESAEADHKDFKEYFORMAT']._serialized_start=785
  _globals['_ECIESAEADHKDFKEYFORMAT']._serialized_end=866
# @@protoc_insertion_point(module_scope)
