# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/aes_gcm_hkdf_streaming.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/aes_gcm_hkdf_streaming.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tink.proto import common_pb2 as tink_dot_proto_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'tink/proto/aes_gcm_hkdf_streaming.proto\x12\x12google.crypto.tink\x1a\x17tink/proto/common.proto\"\x8c\x01\n\x19\x41\x65sGcmHkdfStreamingParams\x12\x1f\n\x17\x63iphertext_segment_size\x18\x01 \x01(\r\x12\x18\n\x10\x64\x65rived_key_size\x18\x02 \x01(\r\x12\x34\n\x0ehkdf_hash_type\x18\x03 \x01(\x0e\x32\x1c.google.crypto.tink.HashType\"\x80\x01\n\x1c\x41\x65sGcmHkdfStreamingKeyFormat\x12\x0f\n\x07version\x18\x03 \x01(\r\x12=\n\x06params\x18\x01 \x01(\x0b\x32-.google.crypto.tink.AesGcmHkdfStreamingParams\x12\x10\n\x08key_size\x18\x02 \x01(\r\"{\n\x16\x41\x65sGcmHkdfStreamingKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12=\n\x06params\x18\x02 \x01(\x0b\x32-.google.crypto.tink.AesGcmHkdfStreamingParams\x12\x11\n\tkey_value\x18\x03 \x01(\x0c\x42i\n\x1c\x63om.google.crypto.tink.protoP\x01ZGgithub.com/tink-crypto/tink-go/v2/proto/aes_gcm_hkdf_streaming_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.aes_gcm_hkdf_streaming_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001ZGgithub.com/tink-crypto/tink-go/v2/proto/aes_gcm_hkdf_streaming_go_proto'
  _globals['_AESGCMHKDFSTREAMINGPARAMS']._serialized_start=89
  _globals['_AESGCMHKDFSTREAMINGPARAMS']._serialized_end=229
  _globals['_AESGCMHKDFSTREAMINGKEYFORMAT']._serialized_start=232
  _globals['_AESGCMHKDFSTREAMINGKEYFORMAT']._serialized_end=360
  _globals['_AESGCMHKDFSTREAMINGKEY']._serialized_start=362
  _globals['_AESGCMHKDFSTREAMINGKEY']._serialized_end=485
# @@protoc_insertion_point(module_scope)
