# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/jwt_hmac.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/jwt_hmac.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19tink/proto/jwt_hmac.proto\x12\x12google.crypto.tink\"\xc3\x01\n\nJwtHmacKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x37\n\talgorithm\x18\x02 \x01(\x0e\x32$.google.crypto.tink.JwtHmacAlgorithm\x12\x11\n\tkey_value\x18\x03 \x01(\x0c\x12<\n\ncustom_kid\x18\x04 \x01(\x0b\x32(.google.crypto.tink.JwtHmacKey.CustomKid\x1a\x1a\n\tCustomKid\x12\r\n\x05value\x18\x01 \x01(\t\"n\n\x10JwtHmacKeyFormat\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x37\n\talgorithm\x18\x02 \x01(\x0e\x32$.google.crypto.tink.JwtHmacAlgorithm\x12\x10\n\x08key_size\x18\x03 \x01(\r*C\n\x10JwtHmacAlgorithm\x12\x0e\n\nHS_UNKNOWN\x10\x00\x12\t\n\x05HS256\x10\x01\x12\t\n\x05HS384\x10\x02\x12\t\n\x05HS512\x10\x03\x42[\n\x1c\x63om.google.crypto.tink.protoP\x01Z9github.com/tink-crypto/tink-go/v2/proto/jwt_hmac_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.jwt_hmac_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z9github.com/tink-crypto/tink-go/v2/proto/jwt_hmac_go_proto'
  _globals['_JWTHMACALGORITHM']._serialized_start=359
  _globals['_JWTHMACALGORITHM']._serialized_end=426
  _globals['_JWTHMACKEY']._serialized_start=50
  _globals['_JWTHMACKEY']._serialized_end=245
  _globals['_JWTHMACKEY_CUSTOMKID']._serialized_start=219
  _globals['_JWTHMACKEY_CUSTOMKID']._serialized_end=245
  _globals['_JWTHMACKEYFORMAT']._serialized_start=247
  _globals['_JWTHMACKEYFORMAT']._serialized_end=357
# @@protoc_insertion_point(module_scope)
