# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/prf_based_deriver.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/prf_based_deriver.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tink.proto import tink_pb2 as tink_dot_proto_dot_tink__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"tink/proto/prf_based_deriver.proto\x12\x12google.crypto.tink\x1a\x15tink/proto/tink.proto\"V\n\x15PrfBasedDeriverParams\x12=\n\x14\x64\x65rived_key_template\x18\x01 \x01(\x0b\x32\x1f.google.crypto.tink.KeyTemplate\"\x90\x01\n\x18PrfBasedDeriverKeyFormat\x12\x39\n\x10prf_key_template\x18\x01 \x01(\x0b\x32\x1f.google.crypto.tink.KeyTemplate\x12\x39\n\x06params\x18\x02 \x01(\x0b\x32).google.crypto.tink.PrfBasedDeriverParams\"\x8e\x01\n\x12PrfBasedDeriverKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12,\n\x07prf_key\x18\x02 \x01(\x0b\x32\x1b.google.crypto.tink.KeyData\x12\x39\n\x06params\x18\x03 \x01(\x0b\x32).google.crypto.tink.PrfBasedDeriverParamsBd\n\x1c\x63om.google.crypto.tink.protoP\x01ZBgithub.com/tink-crypto/tink-go/v2/proto/prf_based_deriver_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.prf_based_deriver_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001ZBgithub.com/tink-crypto/tink-go/v2/proto/prf_based_deriver_go_proto'
  _globals['_PRFBASEDDERIVERPARAMS']._serialized_start=81
  _globals['_PRFBASEDDERIVERPARAMS']._serialized_end=167
  _globals['_PRFBASEDDERIVERKEYFORMAT']._serialized_start=170
  _globals['_PRFBASEDDERIVERKEYFORMAT']._serialized_end=314
  _globals['_PRFBASEDDERIVERKEY']._serialized_start=317
  _globals['_PRFBASEDDERIVERKEY']._serialized_end=459
# @@protoc_insertion_point(module_scope)
