// Copyright 2021 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
////////////////////////////////////////////////////////////////////////////////
syntax = "proto3";

package google.crypto.tink;

option java_package = "com.google.crypto.tink.proto";
option java_multiple_files = true;

message TestProto {
  uint64 num = 1;
  bytes str = 2;  // Placeholder for ctype.
}

message NestedTestProto {
  TestProto a = 1;
  TestProto b = 2;
  uint64 num = 3;
  bytes str = 4;  // Placeholder for ctype.
}

message TestProtoWithoutCtype {
  bytes str = 1;
}
