# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/jwt_rsa_ssa_pss.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/jwt_rsa_ssa_pss.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n tink/proto/jwt_rsa_ssa_pss.proto\x12\x12google.crypto.tink\"\xe1\x01\n\x15JwtRsaSsaPssPublicKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12<\n\talgorithm\x18\x02 \x01(\x0e\x32).google.crypto.tink.JwtRsaSsaPssAlgorithm\x12\t\n\x01n\x18\x03 \x01(\x0c\x12\t\n\x01\x65\x18\x04 \x01(\x0c\x12G\n\ncustom_kid\x18\x05 \x01(\x0b\x32\x33.google.crypto.tink.JwtRsaSsaPssPublicKey.CustomKid\x1a\x1a\n\tCustomKid\x12\r\n\x05value\x18\x01 \x01(\t\"\xae\x01\n\x16JwtRsaSsaPssPrivateKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12=\n\npublic_key\x18\x02 \x01(\x0b\x32).google.crypto.tink.JwtRsaSsaPssPublicKey\x12\t\n\x01\x64\x18\x03 \x01(\x0c\x12\t\n\x01p\x18\x04 \x01(\x0c\x12\t\n\x01q\x18\x05 \x01(\x0c\x12\n\n\x02\x64p\x18\x06 \x01(\x0c\x12\n\n\x02\x64q\x18\x07 \x01(\x0c\x12\x0b\n\x03\x63rt\x18\x08 \x01(\x0c\"\x9d\x01\n\x15JwtRsaSsaPssKeyFormat\x12\x0f\n\x07version\x18\x01 \x01(\r\x12<\n\talgorithm\x18\x02 \x01(\x0e\x32).google.crypto.tink.JwtRsaSsaPssAlgorithm\x12\x1c\n\x14modulus_size_in_bits\x18\x03 \x01(\r\x12\x17\n\x0fpublic_exponent\x18\x04 \x01(\x0c*H\n\x15JwtRsaSsaPssAlgorithm\x12\x0e\n\nPS_UNKNOWN\x10\x00\x12\t\n\x05PS256\x10\x01\x12\t\n\x05PS384\x10\x02\x12\t\n\x05PS512\x10\x03\x42\x62\n\x1c\x63om.google.crypto.tink.protoP\x01Z@github.com/tink-crypto/tink-go/v2/proto/jwt_rsa_ssa_pss_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.jwt_rsa_ssa_pss_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z@github.com/tink-crypto/tink-go/v2/proto/jwt_rsa_ssa_pss_go_proto'
  _globals['_JWTRSASSAPSSALGORITHM']._serialized_start=621
  _globals['_JWTRSASSAPSSALGORITHM']._serialized_end=693
  _globals['_JWTRSASSAPSSPUBLICKEY']._serialized_start=57
  _globals['_JWTRSASSAPSSPUBLICKEY']._serialized_end=282
  _globals['_JWTRSASSAPSSPUBLICKEY_CUSTOMKID']._serialized_start=256
  _globals['_JWTRSASSAPSSPUBLICKEY_CUSTOMKID']._serialized_end=282
  _globals['_JWTRSASSAPSSPRIVATEKEY']._serialized_start=285
  _globals['_JWTRSASSAPSSPRIVATEKEY']._serialized_end=459
  _globals['_JWTRSASSAPSSKEYFORMAT']._serialized_start=462
  _globals['_JWTRSASSAPSSKEYFORMAT']._serialized_end=619
# @@protoc_insertion_point(module_scope)
