# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/aes_ctr_hmac_aead.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/aes_ctr_hmac_aead.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tink.proto import aes_ctr_pb2 as tink_dot_proto_dot_aes__ctr__pb2
from tink.proto import hmac_pb2 as tink_dot_proto_dot_hmac__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"tink/proto/aes_ctr_hmac_aead.proto\x12\x12google.crypto.tink\x1a\x18tink/proto/aes_ctr.proto\x1a\x15tink/proto/hmac.proto\"\x96\x01\n\x17\x41\x65sCtrHmacAeadKeyFormat\x12?\n\x12\x61\x65s_ctr_key_format\x18\x01 \x01(\x0b\x32#.google.crypto.tink.AesCtrKeyFormat\x12:\n\x0fhmac_key_format\x18\x02 \x01(\x0b\x32!.google.crypto.tink.HmacKeyFormat\"\x87\x01\n\x11\x41\x65sCtrHmacAeadKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x32\n\x0b\x61\x65s_ctr_key\x18\x02 \x01(\x0b\x32\x1d.google.crypto.tink.AesCtrKey\x12-\n\x08hmac_key\x18\x03 \x01(\x0b\x32\x1b.google.crypto.tink.HmacKeyBd\n\x1c\x63om.google.crypto.tink.protoP\x01ZBgithub.com/tink-crypto/tink-go/v2/proto/aes_ctr_hmac_aead_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.aes_ctr_hmac_aead_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001ZBgithub.com/tink-crypto/tink-go/v2/proto/aes_ctr_hmac_aead_go_proto'
  _globals['_AESCTRHMACAEADKEYFORMAT']._serialized_start=108
  _globals['_AESCTRHMACAEADKEYFORMAT']._serialized_end=258
  _globals['_AESCTRHMACAEADKEY']._serialized_start=261
  _globals['_AESCTRHMACAEADKEY']._serialized_end=396
# @@protoc_insertion_point(module_scope)
