# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/config.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/config.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17tink/proto/config.proto\x12\x12google.crypto.tink\"\x8a\x01\n\x0cKeyTypeEntry\x12\x16\n\x0eprimitive_name\x18\x01 \x01(\t\x12\x10\n\x08type_url\x18\x02 \x01(\t\x12\x1b\n\x13key_manager_version\x18\x03 \x01(\r\x12\x17\n\x0fnew_key_allowed\x18\x04 \x01(\x08\x12\x16\n\x0e\x63\x61talogue_name\x18\x05 \x01(\t:\x02\x18\x01\"Z\n\x0eRegistryConfig\x12\x13\n\x0b\x63onfig_name\x18\x01 \x01(\t\x12/\n\x05\x65ntry\x18\x02 \x03(\x0b\x32 .google.crypto.tink.KeyTypeEntry:\x02\x18\x01\x42Y\n\x1c\x63om.google.crypto.tink.protoP\x01Z7github.com/tink-crypto/tink-go/v2/proto/config_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.config_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z7github.com/tink-crypto/tink-go/v2/proto/config_go_proto'
  _globals['_KEYTYPEENTRY']._loaded_options = None
  _globals['_KEYTYPEENTRY']._serialized_options = b'\030\001'
  _globals['_REGISTRYCONFIG']._loaded_options = None
  _globals['_REGISTRYCONFIG']._serialized_options = b'\030\001'
  _globals['_KEYTYPEENTRY']._serialized_start=48
  _globals['_KEYTYPEENTRY']._serialized_end=186
  _globals['_REGISTRYCONFIG']._serialized_start=188
  _globals['_REGISTRYCONFIG']._serialized_end=278
# @@protoc_insertion_point(module_scope)
