// Copyright 2017 Google Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
////////////////////////////////////////////////////////////////////////////////

syntax = "proto3";

package google.crypto.tink;

import "tink/proto/common.proto";
import "tink/proto/hmac.proto";

option java_package = "com.google.crypto.tink.proto";
option java_multiple_files = true;
option go_package = "github.com/tink-crypto/tink-go/v2/proto/aes_ctr_hmac_streaming_go_proto";

message AesCtrHmacStreamingParams {
  uint32 ciphertext_segment_size = 1;
  uint32 derived_key_size = 2;  // size of AES-CTR keys derived for each segment
  HashType hkdf_hash_type = 3;  // hash function for key derivation via HKDF
  HmacParams hmac_params = 4;   // params for authentication tags
}

message AesCtrHmacStreamingKeyFormat {
  uint32 version = 3;
  AesCtrHmacStreamingParams params = 1;
  uint32 key_size = 2;  // size of the main key (aka. "ikm", input key material)
}

// key_type: type.googleapis.com/google.crypto.tink.AesCtrHmacStreamingKey
message AesCtrHmacStreamingKey {
  uint32 version = 1;
  AesCtrHmacStreamingParams params = 2;
  bytes key_value = 3 ;  // Placeholder for multi-line ctype and debug_redact.  // the main key, aka. "ikm", input key material
}
