# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/hpke.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/hpke.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15tink/proto/hpke.proto\x12\x12google.crypto.tink\"\x8c\x01\n\nHpkeParams\x12(\n\x03kem\x18\x01 \x01(\x0e\x32\x1b.google.crypto.tink.HpkeKem\x12(\n\x03kdf\x18\x02 \x01(\x0e\x32\x1b.google.crypto.tink.HpkeKdf\x12*\n\x04\x61\x65\x61\x64\x18\x03 \x01(\x0e\x32\x1c.google.crypto.tink.HpkeAead\"d\n\rHpkePublicKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12.\n\x06params\x18\x02 \x01(\x0b\x32\x1e.google.crypto.tink.HpkeParams\x12\x12\n\npublic_key\x18\x03 \x01(\x0c\"m\n\x0eHpkePrivateKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x35\n\npublic_key\x18\x02 \x01(\x0b\x32!.google.crypto.tink.HpkePublicKey\x12\x13\n\x0bprivate_key\x18\x03 \x01(\x0c\"?\n\rHpkeKeyFormat\x12.\n\x06params\x18\x01 \x01(\x0b\x32\x1e.google.crypto.tink.HpkeParams*\xb7\x01\n\x07HpkeKem\x12\x0f\n\x0bKEM_UNKNOWN\x10\x00\x12\x1c\n\x18\x44HKEM_X25519_HKDF_SHA256\x10\x01\x12\x1a\n\x16\x44HKEM_P256_HKDF_SHA256\x10\x02\x12\x1a\n\x16\x44HKEM_P384_HKDF_SHA384\x10\x03\x12\x1a\n\x16\x44HKEM_P521_HKDF_SHA512\x10\x04\x12\n\n\x06X_WING\x10\x05\x12\r\n\tML_KEM768\x10\x06\x12\x0e\n\nML_KEM1024\x10\x07*M\n\x07HpkeKdf\x12\x0f\n\x0bKDF_UNKNOWN\x10\x00\x12\x0f\n\x0bHKDF_SHA256\x10\x01\x12\x0f\n\x0bHKDF_SHA384\x10\x02\x12\x0f\n\x0bHKDF_SHA512\x10\x03*U\n\x08HpkeAead\x12\x10\n\x0c\x41\x45\x41\x44_UNKNOWN\x10\x00\x12\x0f\n\x0b\x41\x45S_128_GCM\x10\x01\x12\x0f\n\x0b\x41\x45S_256_GCM\x10\x02\x12\x15\n\x11\x43HACHA20_POLY1305\x10\x03\x42T\n\x1c\x63om.google.crypto.tink.protoP\x01Z2github.com/tink-crypto/tink-go/v2/proto/hpke_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.hpke_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z2github.com/tink-crypto/tink-go/v2/proto/hpke_proto'
  _globals['_HPKEKEM']._serialized_start=467
  _globals['_HPKEKEM']._serialized_end=650
  _globals['_HPKEKDF']._serialized_start=652
  _globals['_HPKEKDF']._serialized_end=729
  _globals['_HPKEAEAD']._serialized_start=731
  _globals['_HPKEAEAD']._serialized_end=816
  _globals['_HPKEPARAMS']._serialized_start=46
  _globals['_HPKEPARAMS']._serialized_end=186
  _globals['_HPKEPUBLICKEY']._serialized_start=188
  _globals['_HPKEPUBLICKEY']._serialized_end=288
  _globals['_HPKEPRIVATEKEY']._serialized_start=290
  _globals['_HPKEPRIVATEKEY']._serialized_end=399
  _globals['_HPKEKEYFORMAT']._serialized_start=401
  _globals['_HPKEKEYFORMAT']._serialized_end=464
# @@protoc_insertion_point(module_scope)
