# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/ecdsa.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/ecdsa.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tink.proto import common_pb2 as tink_dot_proto_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16tink/proto/ecdsa.proto\x12\x12google.crypto.tink\x1a\x17tink/proto/common.proto\"\xb2\x01\n\x0b\x45\x63\x64saParams\x12/\n\thash_type\x18\x01 \x01(\x0e\x32\x1c.google.crypto.tink.HashType\x12\x34\n\x05\x63urve\x18\x02 \x01(\x0e\x32%.google.crypto.tink.EllipticCurveType\x12<\n\x08\x65ncoding\x18\x03 \x01(\x0e\x32*.google.crypto.tink.EcdsaSignatureEncoding\"h\n\x0e\x45\x63\x64saPublicKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12/\n\x06params\x18\x02 \x01(\x0b\x32\x1f.google.crypto.tink.EcdsaParams\x12\t\n\x01x\x18\x03 \x01(\x0c\x12\t\n\x01y\x18\x04 \x01(\x0c\"m\n\x0f\x45\x63\x64saPrivateKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x36\n\npublic_key\x18\x02 \x01(\x0b\x32\".google.crypto.tink.EcdsaPublicKey\x12\x11\n\tkey_value\x18\x03 \x01(\x0c\"R\n\x0e\x45\x63\x64saKeyFormat\x12/\n\x06params\x18\x02 \x01(\x0b\x32\x1f.google.crypto.tink.EcdsaParams\x12\x0f\n\x07version\x18\x03 \x01(\r*G\n\x16\x45\x63\x64saSignatureEncoding\x12\x14\n\x10UNKNOWN_ENCODING\x10\x00\x12\x0e\n\nIEEE_P1363\x10\x01\x12\x07\n\x03\x44\x45R\x10\x02\x42X\n\x1c\x63om.google.crypto.tink.protoP\x01Z6github.com/tink-crypto/tink-go/v2/proto/ecdsa_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.ecdsa_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z6github.com/tink-crypto/tink-go/v2/proto/ecdsa_go_proto'
  _globals['_ECDSASIGNATUREENCODING']._serialized_start=553
  _globals['_ECDSASIGNATUREENCODING']._serialized_end=624
  _globals['_ECDSAPARAMS']._serialized_start=72
  _globals['_ECDSAPARAMS']._serialized_end=250
  _globals['_ECDSAPUBLICKEY']._serialized_start=252
  _globals['_ECDSAPUBLICKEY']._serialized_end=356
  _globals['_ECDSAPRIVATEKEY']._serialized_start=358
  _globals['_ECDSAPRIVATEKEY']._serialized_end=467
  _globals['_ECDSAKEYFORMAT']._serialized_start=469
  _globals['_ECDSAKEYFORMAT']._serialized_end=551
# @@protoc_insertion_point(module_scope)
