# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/ml_dsa.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/ml_dsa.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17tink/proto/ml_dsa.proto\x12\x12google.crypto.tink\"I\n\x0bMlDsaParams\x12:\n\x0fml_dsa_instance\x18\x01 \x01(\x0e\x32!.google.crypto.tink.MlDsaInstance\"R\n\x0eMlDsaKeyFormat\x12\x0f\n\x07version\x18\x01 \x01(\r\x12/\n\x06params\x18\x02 \x01(\x0b\x32\x1f.google.crypto.tink.MlDsaParams\"e\n\x0eMlDsaPublicKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x11\n\tkey_value\x18\x02 \x01(\x0c\x12/\n\x06params\x18\x03 \x01(\x0b\x32\x1f.google.crypto.tink.MlDsaParams\"m\n\x0fMlDsaPrivateKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x11\n\tkey_value\x18\x02 \x01(\x0c\x12\x36\n\npublic_key\x18\x03 \x01(\x0b\x32\".google.crypto.tink.MlDsaPublicKey*J\n\rMlDsaInstance\x12\x1b\n\x17ML_DSA_UNKNOWN_INSTANCE\x10\x00\x12\r\n\tML_DSA_65\x10\x01\x12\r\n\tML_DSA_87\x10\x02\x42V\n\x1c\x63om.google.crypto.tink.protoP\x01Z4github.com/tink-crypto/tink-go/v2/proto/ml_dsa_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.ml_dsa_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z4github.com/tink-crypto/tink-go/v2/proto/ml_dsa_proto'
  _globals['_MLDSAINSTANCE']._serialized_start=420
  _globals['_MLDSAINSTANCE']._serialized_end=494
  _globals['_MLDSAPARAMS']._serialized_start=47
  _globals['_MLDSAPARAMS']._serialized_end=120
  _globals['_MLDSAKEYFORMAT']._serialized_start=122
  _globals['_MLDSAKEYFORMAT']._serialized_end=204
  _globals['_MLDSAPUBLICKEY']._serialized_start=206
  _globals['_MLDSAPUBLICKEY']._serialized_end=307
  _globals['_MLDSAPRIVATEKEY']._serialized_start=309
  _globals['_MLDSAPRIVATEKEY']._serialized_end=418
# @@protoc_insertion_point(module_scope)
