# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/x_aes_gcm.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/x_aes_gcm.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1atink/proto/x_aes_gcm.proto\x12\x12google.crypto.tink\"\"\n\rXAesGcmParams\x12\x11\n\tsalt_size\x18\x01 \x01(\r\"\\\n\x10XAesGcmKeyFormat\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x31\n\x06params\x18\x03 \x01(\x0b\x32!.google.crypto.tink.XAesGcmParamsJ\x04\x08\x02\x10\x03\"c\n\nXAesGcmKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x31\n\x06params\x18\x02 \x01(\x0b\x32!.google.crypto.tink.XAesGcmParams\x12\x11\n\tkey_value\x18\x03 \x01(\x0c\x42\\\n\x1c\x63om.google.crypto.tink.protoP\x01Z:github.com/tink-crypto/tink-go/v2/proto/x_aes_gcm_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.x_aes_gcm_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z:github.com/tink-crypto/tink-go/v2/proto/x_aes_gcm_go_proto'
  _globals['_XAESGCMPARAMS']._serialized_start=50
  _globals['_XAESGCMPARAMS']._serialized_end=84
  _globals['_XAESGCMKEYFORMAT']._serialized_start=86
  _globals['_XAESGCMKEYFORMAT']._serialized_end=178
  _globals['_XAESGCMKEY']._serialized_start=180
  _globals['_XAESGCMKEY']._serialized_end=279
# @@protoc_insertion_point(module_scope)
