# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/slh_dsa.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/slh_dsa.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18tink/proto/slh_dsa.proto\x12\x12google.crypto.tink\"\x92\x01\n\x0cSlhDsaParams\x12\x10\n\x08key_size\x18\x01 \x01(\x05\x12\x35\n\thash_type\x18\x02 \x01(\x0e\x32\".google.crypto.tink.SlhDsaHashType\x12\x39\n\x08sig_type\x18\x03 \x01(\x0e\x32\'.google.crypto.tink.SlhDsaSignatureType\"T\n\x0fSlhDsaKeyFormat\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x30\n\x06params\x18\x02 \x01(\x0b\x32 .google.crypto.tink.SlhDsaParams\"g\n\x0fSlhDsaPublicKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x11\n\tkey_value\x18\x02 \x01(\x0c\x12\x30\n\x06params\x18\x03 \x01(\x0b\x32 .google.crypto.tink.SlhDsaParams\"o\n\x10SlhDsaPrivateKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x11\n\tkey_value\x18\x02 \x01(\x0c\x12\x37\n\npublic_key\x18\x03 \x01(\x0b\x32#.google.crypto.tink.SlhDsaPublicKey*H\n\x0eSlhDsaHashType\x12!\n\x1dSLH_DSA_HASH_TYPE_UNSPECIFIED\x10\x00\x12\x08\n\x04SHA2\x10\x01\x12\t\n\x05SHAKE\x10\x02*d\n\x13SlhDsaSignatureType\x12&\n\"SLH_DSA_SIGNATURE_TYPE_UNSPECIFIED\x10\x00\x12\x10\n\x0c\x46\x41ST_SIGNING\x10\x01\x12\x13\n\x0fSMALL_SIGNATURE\x10\x02\x42W\n\x1c\x63om.google.crypto.tink.protoP\x01Z5github.com/tink-crypto/tink-go/v2/proto/slh_dsa_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.slh_dsa_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z5github.com/tink-crypto/tink-go/v2/proto/slh_dsa_proto'
  _globals['_SLHDSAHASHTYPE']._serialized_start=501
  _globals['_SLHDSAHASHTYPE']._serialized_end=573
  _globals['_SLHDSASIGNATURETYPE']._serialized_start=575
  _globals['_SLHDSASIGNATURETYPE']._serialized_end=675
  _globals['_SLHDSAPARAMS']._serialized_start=49
  _globals['_SLHDSAPARAMS']._serialized_end=195
  _globals['_SLHDSAKEYFORMAT']._serialized_start=197
  _globals['_SLHDSAKEYFORMAT']._serialized_end=281
  _globals['_SLHDSAPUBLICKEY']._serialized_start=283
  _globals['_SLHDSAPUBLICKEY']._serialized_end=386
  _globals['_SLHDSAPRIVATEKEY']._serialized_start=388
  _globals['_SLHDSAPRIVATEKEY']._serialized_end=499
# @@protoc_insertion_point(module_scope)
