// Copyright 2024 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
///////////////////////////////////////////////////////////////////////////////
syntax = "proto3";

package google.crypto.tink;

option java_package = "com.google.crypto.tink.proto";
option java_multiple_files = true;
option go_package = "github.com/tink-crypto/tink-go/v2/proto/slh_dsa_proto";

enum SlhDsaHashType {
  SLH_DSA_HASH_TYPE_UNSPECIFIED = 0;
  SHA2 = 1;
  SHAKE = 2;
}

enum SlhDsaSignatureType {
  SLH_DSA_SIGNATURE_TYPE_UNSPECIFIED = 0;
  FAST_SIGNING = 1;
  SMALL_SIGNATURE = 2;
}

// Protos for the Stateless Hash-Based Digital Signature Algorithm
// https://csrc.nist.gov/pubs/fips/205/final
message SlhDsaParams {
  // Required
  int32 key_size = 1;
  // Required.
  SlhDsaHashType hash_type = 2;
  // Required.
  SlhDsaSignatureType sig_type = 3;
}

message SlhDsaKeyFormat {
  uint32 version = 1;
  // Required.
  SlhDsaParams params = 2;
}

// key_type: type.googleapis.com/google.crypto.tink.SlhDsaPublicKey
message SlhDsaPublicKey {
  // Required.
  uint32 version = 1;
  // Required.
  bytes key_value = 2;  // Placeholder for ctype.
  // Required
  SlhDsaParams params = 3;
}

// key_type: type.googleapis.com/google.crypto.tink.SlhDsaPrivateKey
message SlhDsaPrivateKey {
  // Required.
  uint32 version = 1;
  // Required.
  bytes key_value = 2;  // Placeholder for ctype.
  // Required. The corresponding public key.
  SlhDsaPublicKey public_key = 3;
}
