# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/tink.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/tink.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15tink/proto/tink.proto\x12\x12google.crypto.tink\"p\n\x0bKeyTemplate\x12\x10\n\x08type_url\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12@\n\x12output_prefix_type\x18\x03 \x01(\x0e\x32$.google.crypto.tink.OutputPrefixType\"\xe8\x01\n\x07KeyData\x12\x10\n\x08type_url\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x0c\x12\x46\n\x11key_material_type\x18\x03 \x01(\x0e\x32+.google.crypto.tink.KeyData.KeyMaterialType\"t\n\x0fKeyMaterialType\x12\x17\n\x13UNKNOWN_KEYMATERIAL\x10\x00\x12\r\n\tSYMMETRIC\x10\x01\x12\x16\n\x12\x41SYMMETRIC_PRIVATE\x10\x02\x12\x15\n\x11\x41SYMMETRIC_PUBLIC\x10\x03\x12\n\n\x06REMOTE\x10\x04\"\x89\x02\n\x06Keyset\x12\x16\n\x0eprimary_key_id\x18\x01 \x01(\r\x12+\n\x03key\x18\x02 \x03(\x0b\x32\x1e.google.crypto.tink.Keyset.Key\x1a\xb9\x01\n\x03Key\x12-\n\x08key_data\x18\x01 \x01(\x0b\x32\x1b.google.crypto.tink.KeyData\x12\x31\n\x06status\x18\x02 \x01(\x0e\x32!.google.crypto.tink.KeyStatusType\x12\x0e\n\x06key_id\x18\x03 \x01(\r\x12@\n\x12output_prefix_type\x18\x04 \x01(\x0e\x32$.google.crypto.tink.OutputPrefixType\"\x81\x02\n\nKeysetInfo\x12\x16\n\x0eprimary_key_id\x18\x01 \x01(\r\x12\x38\n\x08key_info\x18\x02 \x03(\x0b\x32&.google.crypto.tink.KeysetInfo.KeyInfo\x1a\xa0\x01\n\x07KeyInfo\x12\x10\n\x08type_url\x18\x01 \x01(\t\x12\x31\n\x06status\x18\x02 \x01(\x0e\x32!.google.crypto.tink.KeyStatusType\x12\x0e\n\x06key_id\x18\x03 \x01(\r\x12@\n\x12output_prefix_type\x18\x04 \x01(\x0e\x32$.google.crypto.tink.OutputPrefixType\"`\n\x0f\x45ncryptedKeyset\x12\x18\n\x10\x65ncrypted_keyset\x18\x02 \x01(\x0c\x12\x33\n\x0bkeyset_info\x18\x03 \x01(\x0b\x32\x1e.google.crypto.tink.KeysetInfo*M\n\rKeyStatusType\x12\x12\n\x0eUNKNOWN_STATUS\x10\x00\x12\x0b\n\x07\x45NABLED\x10\x01\x12\x0c\n\x08\x44ISABLED\x10\x02\x12\r\n\tDESTROYED\x10\x03*k\n\x10OutputPrefixType\x12\x12\n\x0eUNKNOWN_PREFIX\x10\x00\x12\x08\n\x04TINK\x10\x01\x12\n\n\x06LEGACY\x10\x02\x12\x07\n\x03RAW\x10\x03\x12\x0b\n\x07\x43RUNCHY\x10\x04\x12\x17\n\x13WITH_ID_REQUIREMENT\x10\x05\x42`\n\x1c\x63om.google.crypto.tink.protoP\x01Z5github.com/tink-crypto/tink-go/v2/proto/tink_go_proto\xa2\x02\x06TINKPBb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.tink_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z5github.com/tink-crypto/tink-go/v2/proto/tink_go_proto\242\002\006TINKPB'
  _globals['_KEYSTATUSTYPE']._serialized_start=1020
  _globals['_KEYSTATUSTYPE']._serialized_end=1097
  _globals['_OUTPUTPREFIXTYPE']._serialized_start=1099
  _globals['_OUTPUTPREFIXTYPE']._serialized_end=1206
  _globals['_KEYTEMPLATE']._serialized_start=45
  _globals['_KEYTEMPLATE']._serialized_end=157
  _globals['_KEYDATA']._serialized_start=160
  _globals['_KEYDATA']._serialized_end=392
  _globals['_KEYDATA_KEYMATERIALTYPE']._serialized_start=276
  _globals['_KEYDATA_KEYMATERIALTYPE']._serialized_end=392
  _globals['_KEYSET']._serialized_start=395
  _globals['_KEYSET']._serialized_end=660
  _globals['_KEYSET_KEY']._serialized_start=475
  _globals['_KEYSET_KEY']._serialized_end=660
  _globals['_KEYSETINFO']._serialized_start=663
  _globals['_KEYSETINFO']._serialized_end=920
  _globals['_KEYSETINFO_KEYINFO']._serialized_start=760
  _globals['_KEYSETINFO_KEYINFO']._serialized_end=920
  _globals['_ENCRYPTEDKEYSET']._serialized_start=922
  _globals['_ENCRYPTEDKEYSET']._serialized_end=1018
# @@protoc_insertion_point(module_scope)
