# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/test_proto.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/test_proto.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1btink/proto/test_proto.proto\x12\x12google.crypto.tink\"%\n\tTestProto\x12\x0b\n\x03num\x18\x01 \x01(\x04\x12\x0b\n\x03str\x18\x02 \x01(\x0c\"\x7f\n\x0fNestedTestProto\x12(\n\x01\x61\x18\x01 \x01(\x0b\x32\x1d.google.crypto.tink.TestProto\x12(\n\x01\x62\x18\x02 \x01(\x0b\x32\x1d.google.crypto.tink.TestProto\x12\x0b\n\x03num\x18\x03 \x01(\x04\x12\x0b\n\x03str\x18\x04 \x01(\x0c\"$\n\x15TestProtoWithoutCtype\x12\x0b\n\x03str\x18\x01 \x01(\x0c\x42 \n\x1c\x63om.google.crypto.tink.protoP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.test_proto_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001'
  _globals['_TESTPROTO']._serialized_start=51
  _globals['_TESTPROTO']._serialized_end=88
  _globals['_NESTEDTESTPROTO']._serialized_start=90
  _globals['_NESTEDTESTPROTO']._serialized_end=217
  _globals['_TESTPROTOWITHOUTCTYPE']._serialized_start=219
  _globals['_TESTPROTOWITHOUTCTYPE']._serialized_end=255
# @@protoc_insertion_point(module_scope)
