# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/rsa_ssa_pkcs1.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/rsa_ssa_pkcs1.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tink.proto import common_pb2 as tink_dot_proto_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1etink/proto/rsa_ssa_pkcs1.proto\x12\x12google.crypto.tink\x1a\x17tink/proto/common.proto\"D\n\x11RsaSsaPkcs1Params\x12/\n\thash_type\x18\x01 \x01(\x0e\x32\x1c.google.crypto.tink.HashType\"t\n\x14RsaSsaPkcs1PublicKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x35\n\x06params\x18\x02 \x01(\x0b\x32%.google.crypto.tink.RsaSsaPkcs1Params\x12\t\n\x01n\x18\x03 \x01(\x0c\x12\t\n\x01\x65\x18\x04 \x01(\x0c\"\xac\x01\n\x15RsaSsaPkcs1PrivateKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12<\n\npublic_key\x18\x02 \x01(\x0b\x32(.google.crypto.tink.RsaSsaPkcs1PublicKey\x12\t\n\x01\x64\x18\x03 \x01(\x0c\x12\t\n\x01p\x18\x04 \x01(\x0c\x12\t\n\x01q\x18\x05 \x01(\x0c\x12\n\n\x02\x64p\x18\x06 \x01(\x0c\x12\n\n\x02\x64q\x18\x07 \x01(\x0c\x12\x0b\n\x03\x63rt\x18\x08 \x01(\x0c\"\x84\x01\n\x14RsaSsaPkcs1KeyFormat\x12\x35\n\x06params\x18\x01 \x01(\x0b\x32%.google.crypto.tink.RsaSsaPkcs1Params\x12\x1c\n\x14modulus_size_in_bits\x18\x02 \x01(\r\x12\x17\n\x0fpublic_exponent\x18\x03 \x01(\x0c\x42`\n\x1c\x63om.google.crypto.tink.protoP\x01Z>github.com/tink-crypto/tink-go/v2/proto/rsa_ssa_pkcs1_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.rsa_ssa_pkcs1_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z>github.com/tink-crypto/tink-go/v2/proto/rsa_ssa_pkcs1_go_proto'
  _globals['_RSASSAPKCS1PARAMS']._serialized_start=79
  _globals['_RSASSAPKCS1PARAMS']._serialized_end=147
  _globals['_RSASSAPKCS1PUBLICKEY']._serialized_start=149
  _globals['_RSASSAPKCS1PUBLICKEY']._serialized_end=265
  _globals['_RSASSAPKCS1PRIVATEKEY']._serialized_start=268
  _globals['_RSASSAPKCS1PRIVATEKEY']._serialized_end=440
  _globals['_RSASSAPKCS1KEYFORMAT']._serialized_start=443
  _globals['_RSASSAPKCS1KEYFORMAT']._serialized_end=575
# @@protoc_insertion_point(module_scope)
