# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/kms_aead.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/kms_aead.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19tink/proto/kms_aead.proto\x12\x12google.crypto.tink\"#\n\x10KmsAeadKeyFormat\x12\x0f\n\x07key_uri\x18\x01 \x01(\t\"S\n\nKmsAeadKey\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x34\n\x06params\x18\x02 \x01(\x0b\x32$.google.crypto.tink.KmsAeadKeyFormatB[\n\x1c\x63om.google.crypto.tink.protoP\x01Z9github.com/tink-crypto/tink-go/v2/proto/kms_aead_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.kms_aead_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z9github.com/tink-crypto/tink-go/v2/proto/kms_aead_go_proto'
  _globals['_KMSAEADKEYFORMAT']._serialized_start=49
  _globals['_KMSAEADKEYFORMAT']._serialized_end=84
  _globals['_KMSAEADKEY']._serialized_start=86
  _globals['_KMSAEADKEY']._serialized_end=169
# @@protoc_insertion_point(module_scope)
