# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/xchacha20_poly1305.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/xchacha20_poly1305.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#tink/proto/xchacha20_poly1305.proto\x12\x12google.crypto.tink\"-\n\x1aXChaCha20Poly1305KeyFormat\x12\x0f\n\x07version\x18\x01 \x01(\r\":\n\x14XChaCha20Poly1305Key\x12\x0f\n\x07version\x18\x01 \x01(\r\x12\x11\n\tkey_value\x18\x03 \x01(\x0c\x42\x65\n\x1c\x63om.google.crypto.tink.protoP\x01ZCgithub.com/tink-crypto/tink-go/v2/proto/xchacha20_poly1305_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.xchacha20_poly1305_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001ZCgithub.com/tink-crypto/tink-go/v2/proto/xchacha20_poly1305_go_proto'
  _globals['_XCHACHA20POLY1305KEYFORMAT']._serialized_start=59
  _globals['_XCHACHA20POLY1305KEYFORMAT']._serialized_end=104
  _globals['_XCHACHA20POLY1305KEY']._serialized_start=106
  _globals['_XCHACHA20POLY1305KEY']._serialized_end=164
# @@protoc_insertion_point(module_scope)
