# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: tink/proto/common.proto
# Protobuf Python Version: 6.30.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    30,
    2,
    '',
    'tink/proto/common.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17tink/proto/common.proto\x12\x12google.crypto.tink*c\n\x11\x45llipticCurveType\x12\x11\n\rUNKNOWN_CURVE\x10\x00\x12\r\n\tNIST_P256\x10\x02\x12\r\n\tNIST_P384\x10\x03\x12\r\n\tNIST_P521\x10\x04\x12\x0e\n\nCURVE25519\x10\x05*j\n\rEcPointFormat\x12\x12\n\x0eUNKNOWN_FORMAT\x10\x00\x12\x10\n\x0cUNCOMPRESSED\x10\x01\x12\x0e\n\nCOMPRESSED\x10\x02\x12#\n\x1f\x44O_NOT_USE_CRUNCHY_UNCOMPRESSED\x10\x03*V\n\x08HashType\x12\x10\n\x0cUNKNOWN_HASH\x10\x00\x12\x08\n\x04SHA1\x10\x01\x12\n\n\x06SHA384\x10\x02\x12\n\n\x06SHA256\x10\x03\x12\n\n\x06SHA512\x10\x04\x12\n\n\x06SHA224\x10\x05\x42Y\n\x1c\x63om.google.crypto.tink.protoP\x01Z7github.com/tink-crypto/tink-go/v2/proto/common_go_protob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tink.proto.common_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.google.crypto.tink.protoP\001Z7github.com/tink-crypto/tink-go/v2/proto/common_go_proto'
  _globals['_ELLIPTICCURVETYPE']._serialized_start=47
  _globals['_ELLIPTICCURVETYPE']._serialized_end=146
  _globals['_ECPOINTFORMAT']._serialized_start=148
  _globals['_ECPOINTFORMAT']._serialized_end=254
  _globals['_HASHTYPE']._serialized_start=256
  _globals['_HASHTYPE']._serialized_end=342
# @@protoc_insertion_point(module_scope)
