// Copyright 2017 Google Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
////////////////////////////////////////////////////////////////////////////////

// Definitions for Ed25519 Digital Signature Algorithm.
// See https://ed25519.cr.yp.to/ed25519-20110926.pdf and
// https://tools.ietf.org/html/rfc8032.
syntax = "proto3";

package google.crypto.tink;

option java_package = "com.google.crypto.tink.proto";
option java_multiple_files = true;
option go_package = "github.com/tink-crypto/tink-go/v2/proto/ed25519_go_proto";

message Ed25519KeyFormat {
  uint32 version = 1;
}

// key_type: type.googleapis.com/google.crypto.tink.Ed25519PublicKey
message Ed25519PublicKey {
  // Required.
  uint32 version = 1;
  // The public key is 32 bytes, encoded according to
  // https://tools.ietf.org/html/rfc8032#section-5.1.2.
  // Required.
  bytes key_value = 2;  // Placeholder for ctype.
}

// key_type: type.googleapis.com/google.crypto.tink.Ed25519PrivateKey
message Ed25519PrivateKey {
  // Required.
  uint32 version = 1;
  // The private key is 32 bytes of cryptographically secure random data.
  // See https://tools.ietf.org/html/rfc8032#section-5.1.5.
  // Required.
  bytes key_value = 2;  // Placeholder for ctype and debug_redact.
  // The corresponding public key.
  Ed25519PublicKey public_key = 3;
}
