// Copyright 2024 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
///////////////////////////////////////////////////////////////////////////////

// Protos for Module-Lattice Digital Signature Algorithm (ML-DSA).
// See https://csrc.nist.gov/pubs/fips/204/final.
syntax = "proto3";

package google.crypto.tink;

option java_package = "com.google.crypto.tink.proto";
option java_multiple_files = true;
option go_package = "github.com/tink-crypto/tink-go/v2/proto/ml_dsa_proto";

// LINT: LEGACY_NAMES
enum MlDsaInstance {
  ML_DSA_UNKNOWN_INSTANCE = 0;
  ML_DSA_65 = 1;
  ML_DSA_87 = 2;
}

message MlDsaParams {
  // Required.
  MlDsaInstance ml_dsa_instance = 1;
}

message MlDsaKeyFormat {
  // Required.
  uint32 version = 1;
  // Required.
  MlDsaParams params = 2;
}

// key_type: type.googleapis.com/google.crypto.tink.MlDsaPublicKey
message MlDsaPublicKey {
  // Required.
  uint32 version = 1;
  // Required.
  bytes key_value = 2;
  // Required.
  MlDsaParams params = 3;
}

// key_type: type.googleapis.com/google.crypto.tink.MlDsaPrivateKey
message MlDsaPrivateKey {
  // Required.
  uint32 version = 1;
  // Required. Note that this contains the seed used to generate the private
  // key, not the private key itself.
  bytes key_value = 2;
  // The corresponding public key.
  MlDsaPublicKey public_key = 3;
}
